@@ -1,4 +1,4 @@
-//===-- SBInstructionList.cpp -----------------------------------*- C++ -*-===//
+//===-- SBInstructionList.cpp ---------------------------------------------===//
 //
 // Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
 // See https://llvm.org/LICENSE.txt for license information.
@@ -42,7 +42,7 @@ operator=(const SBInstructionList &rhs) {
   return LLDB_RECORD_RESULT(*this);
 }
 
-SBInstructionList::~SBInstructionList() {}
+SBInstructionList::~SBInstructionList() = default;
 
 bool SBInstructionList::IsValid() const {
   LLDB_RECORD_METHOD_CONST_NO_ARGS(bool, SBInstructionList, IsValid);
