@@ -1,4 +1,4 @@
-//===-- CommunicationKDP.cpp ------------------------------------*- C++ -*-===//
+//===-- CommunicationKDP.cpp ----------------------------------------------===//
 //
 // Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
 // See https://llvm.org/LICENSE.txt for license information.
@@ -8,10 +8,9 @@
 
 #include "CommunicationKDP.h"
 
-#include <errno.h>
-#include <limits.h>
-#include <string.h>
-
+#include <cerrno>
+#include <climits>
+#include <cstring>
 
 #include "lldb/Core/DumpDataExtractor.h"
 #include "lldb/Host/Host.h"
@@ -77,11 +76,9 @@ bool CommunicationKDP::SendRequestAndGetReply(
   }
 
   std::lock_guard<std::recursive_mutex> guard(m_sequence_mutex);
-#ifdef LLDB_CONFIGURATION_DEBUG
   // NOTE: this only works for packets that are in native endian byte order
   assert(request_packet.GetSize() ==
          *((const uint16_t *)(request_packet.GetData() + 2)));
-#endif
   lldb::offset_t offset = 1;
   const uint32_t num_retries = 3;
   for (uint32_t i = 0; i < num_retries; ++i) {
@@ -196,12 +193,11 @@ size_t CommunicationKDP::WaitForPacketWithTimeoutMicroSecondsNoLock(
                                  : std::chrono::microseconds(timeout_usec),
                              status, &error);
 
-    LLDB_LOGV(log, 
-      "Read (buffer, sizeof(buffer), timeout_usec = 0x{0:x}, "
-                  "status = {1}, error = {2}) => bytes_read = {4}",
-                  timeout_usec,
-                  Communication::ConnectionStatusAsCString(status),
-                  error, bytes_read);
+    LLDB_LOGV(log,
+              "Read (buffer, sizeof(buffer), timeout_usec = 0x{0:x}, "
+              "status = {1}, error = {2}) => bytes_read = {4}",
+              timeout_usec, Communication::ConnectionStatusAsString(status),
+              error, bytes_read);
 
     if (bytes_read > 0) {
       if (CheckForPacket(buffer, bytes_read, packet))
@@ -241,6 +237,7 @@ bool CommunicationKDP::CheckForPacket(const uint8_t *src, size_t src_len,
     if (log && log->GetVerbose()) {
       PacketStreamType log_strm;
       DumpHexBytes(&log_strm, src, src_len, UINT32_MAX, LLDB_INVALID_ADDRESS);
+      log_strm.PutChar('\0');
       LLDB_LOGF(log, "CommunicationKDP::%s adding %u bytes: %s", __FUNCTION__,
                 (uint32_t)src_len, log_strm.GetData());
     }
@@ -306,7 +303,7 @@ bool CommunicationKDP::CheckForPacket(const uint8_t *src, size_t src_len,
       if (length <= bytes_available) {
         // We have an entire packet ready, we need to copy the data bytes into
         // a buffer that will be owned by the packet and erase the bytes from
-        // our communcation buffer "m_bytes"
+        // our communication buffer "m_bytes"
         packet.SetData(DataBufferSP(new DataBufferHeap(&m_bytes[0], length)));
         m_bytes.erase(0, length);
 
@@ -447,7 +444,7 @@ lldb_private::UUID CommunicationKDP::GetUUID() {
   if (uuid_str.size() < 32)
     return uuid;
 
-  if (uuid.SetFromStringRef(uuid_str) == 0) {
+  if (!uuid.SetFromStringRef(uuid_str)) {
     UUID invalid_uuid;
     return invalid_uuid;
   }
@@ -788,7 +785,7 @@ void CommunicationKDP::DumpPacket(Stream &s, const DataExtractor &packet) {
           const uint32_t region_count = packet.GetU32(&offset);
           s.Printf(" (count = %u", region_count);
           for (uint32_t i = 0; i < region_count; ++i) {
-            const addr_t region_addr = packet.GetPointer(&offset);
+            const addr_t region_addr = packet.GetAddress(&offset);
             const uint32_t region_size = packet.GetU32(&offset);
             const uint32_t region_prot = packet.GetU32(&offset);
             s.Printf("\n\tregion[%" PRIu64 "] = { range = [0x%16.16" PRIx64
