@@ -1,9 +1,8 @@
-//===-- ThreadPlanCallFunction.cpp ------------------------------*- C++ -*-===//
+//===-- ThreadPlanCallFunction.cpp ----------------------------------------===//
 //
-//                     The LLVM Compiler Infrastructure
-//
-// This file is distributed under the University of Illinois Open Source
-// License. See LICENSE.TXT for details.
+// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
+// See https://llvm.org/LICENSE.txt for license information.
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 //
 //===----------------------------------------------------------------------===//
 
@@ -25,12 +24,12 @@
 #include "lldb/Utility/Log.h"
 #include "lldb/Utility/Stream.h"
 
+#include <memory>
+
 using namespace lldb;
 using namespace lldb_private;
 
-//----------------------------------------------------------------------
 // ThreadPlanCallFunction: Plan to call a single function
-//----------------------------------------------------------------------
 bool ThreadPlanCallFunction::ConstructorSetup(
     Thread &thread, ABI *&abi, lldb::addr_t &start_load_addr,
     lldb::addr_t &function_load_addr) {
@@ -60,46 +59,21 @@ bool ThreadPlanCallFunction::ConstructorSetup(
     m_constructor_errors.Printf(
         "Trying to put the stack in unreadable memory at: 0x%" PRIx64 ".",
         m_function_sp);
-    if (log)
-      log->Printf("ThreadPlanCallFunction(%p): %s.", static_cast<void *>(this),
-                  m_constructor_errors.GetData());
+    LLDB_LOGF(log, "ThreadPlanCallFunction(%p): %s.", static_cast<void *>(this),
+              m_constructor_errors.GetData());
     return false;
   }
 
-  Module *exe_module = GetTarget().GetExecutableModulePointer();
-
-  if (exe_module == nullptr) {
+  llvm::Expected<Address> start_address = GetTarget().GetEntryPointAddress();
+  if (!start_address) {
     m_constructor_errors.Printf(
-        "Can't execute code without an executable module.");
-    if (log)
-      log->Printf("ThreadPlanCallFunction(%p): %s.", static_cast<void *>(this),
-                  m_constructor_errors.GetData());
+        "%s", llvm::toString(start_address.takeError()).c_str());
+    LLDB_LOGF(log, "ThreadPlanCallFunction(%p): %s.", static_cast<void *>(this),
+              m_constructor_errors.GetData());
     return false;
-  } else {
-    ObjectFile *objectFile = exe_module->GetObjectFile();
-    if (!objectFile) {
-      m_constructor_errors.Printf(
-          "Could not find object file for module \"%s\".",
-          exe_module->GetFileSpec().GetFilename().AsCString());
-
-      if (log)
-        log->Printf("ThreadPlanCallFunction(%p): %s.",
-                    static_cast<void *>(this), m_constructor_errors.GetData());
-      return false;
-    }
-
-    m_start_addr = objectFile->GetEntryPointAddress();
-    if (!m_start_addr.IsValid()) {
-      m_constructor_errors.Printf(
-          "Could not find entry point address for executable module \"%s\".",
-          exe_module->GetFileSpec().GetFilename().AsCString());
-      if (log)
-        log->Printf("ThreadPlanCallFunction(%p): %s.",
-                    static_cast<void *>(this), m_constructor_errors.GetData());
-      return false;
-    }
   }
 
+  m_start_addr = *start_address;
   start_load_addr = m_start_addr.GetLoadAddress(&GetTarget());
 
   // Checkpoint the thread state so we can restore it later.
@@ -110,9 +84,8 @@ bool ThreadPlanCallFunction::ConstructorSetup(
   if (!thread.CheckpointThreadState(m_stored_thread_state)) {
     m_constructor_errors.Printf("Setting up ThreadPlanCallFunction, failed to "
                                 "checkpoint thread state.");
-    if (log)
-      log->Printf("ThreadPlanCallFunction(%p): %s.", static_cast<void *>(this),
-                  m_constructor_errors.GetData());
+    LLDB_LOGF(log, "ThreadPlanCallFunction(%p): %s.", static_cast<void *>(this),
+              m_constructor_errors.GetData());
     return false;
   }
   function_load_addr = m_function_addr.GetLoadAddress(&GetTarget());
@@ -173,7 +146,7 @@ void ThreadPlanCallFunction::ReportRegisterState(const char *message) {
   Log *log(lldb_private::GetLogIfAllCategoriesSet(LIBLLDB_LOG_STEP));
   if (log && log->GetVerbose()) {
     StreamString strm;
-    RegisterContext *reg_ctx = m_thread.GetRegisterContext().get();
+    RegisterContext *reg_ctx = GetThread().GetRegisterContext().get();
 
     log->PutCString(message);
 
@@ -197,31 +170,31 @@ void ThreadPlanCallFunction::DoTakedown(bool success) {
 
   if (!m_valid) {
     // Don't call DoTakedown if we were never valid to begin with.
-    if (log)
-      log->Printf("ThreadPlanCallFunction(%p): Log called on "
-                  "ThreadPlanCallFunction that was never valid.",
-                  static_cast<void *>(this));
+    LLDB_LOGF(log,
+              "ThreadPlanCallFunction(%p): Log called on "
+              "ThreadPlanCallFunction that was never valid.",
+              static_cast<void *>(this));
     return;
   }
 
   if (!m_takedown_done) {
+    Thread &thread = GetThread();
     if (success) {
       SetReturnValue();
     }
-    if (log)
-      log->Printf("ThreadPlanCallFunction(%p): DoTakedown called for thread "
-                  "0x%4.4" PRIx64 ", m_valid: %d complete: %d.\n",
-                  static_cast<void *>(this), m_thread.GetID(), m_valid,
-                  IsPlanComplete());
+    LLDB_LOGF(log,
+              "ThreadPlanCallFunction(%p): DoTakedown called for thread "
+              "0x%4.4" PRIx64 ", m_valid: %d complete: %d.\n",
+              static_cast<void *>(this), m_tid, m_valid, IsPlanComplete());
     m_takedown_done = true;
     m_stop_address =
-        m_thread.GetStackFrameAtIndex(0)->GetRegisterContext()->GetPC();
+        thread.GetStackFrameAtIndex(0)->GetRegisterContext()->GetPC();
     m_real_stop_info_sp = GetPrivateStopInfo();
-    if (!m_thread.RestoreRegisterStateFromCheckpoint(m_stored_thread_state)) {
-      if (log)
-        log->Printf("ThreadPlanCallFunction(%p): DoTakedown failed to restore "
-                    "register state",
-                    static_cast<void *>(this));
+    if (!thread.RestoreRegisterStateFromCheckpoint(m_stored_thread_state)) {
+      LLDB_LOGF(log,
+                "ThreadPlanCallFunction(%p): DoTakedown failed to restore "
+                "register state",
+                static_cast<void *>(this));
     }
     SetPlanComplete(success);
     ClearBreakpoints();
@@ -229,11 +202,10 @@ void ThreadPlanCallFunction::DoTakedown(bool success) {
       ReportRegisterState("Restoring thread state after function call.  "
                           "Restored register state:");
   } else {
-    if (log)
-      log->Printf("ThreadPlanCallFunction(%p): DoTakedown called as no-op for "
-                  "thread 0x%4.4" PRIx64 ", m_valid: %d complete: %d.\n",
-                  static_cast<void *>(this), m_thread.GetID(), m_valid,
-                  IsPlanComplete());
+    LLDB_LOGF(log,
+              "ThreadPlanCallFunction(%p): DoTakedown called as no-op for "
+              "thread 0x%4.4" PRIx64 ", m_valid: %d complete: %d.\n",
+              static_cast<void *>(this), m_tid, m_valid, IsPlanComplete());
   }
 }
 
@@ -243,9 +215,8 @@ void ThreadPlanCallFunction::GetDescription(Stream *s, DescriptionLevel level) {
   if (level == eDescriptionLevelBrief) {
     s->Printf("Function call thread plan");
   } else {
-    TargetSP target_sp(m_thread.CalculateTarget());
     s->Printf("Thread plan to call 0x%" PRIx64,
-              m_function_addr.GetLoadAddress(target_sp.get()));
+              m_function_addr.GetLoadAddress(&GetTarget()));
   }
 }
 
@@ -289,10 +260,9 @@ bool ThreadPlanCallFunction::DoPlanExplainsStop(Event *event_ptr) {
     stop_reason = eStopReasonNone;
   else
     stop_reason = m_real_stop_info_sp->GetStopReason();
-  if (log)
-    log->Printf(
-        "ThreadPlanCallFunction::PlanExplainsStop: Got stop reason - %s.",
-        Thread::StopReasonAsCString(stop_reason));
+  LLDB_LOG(log,
+           "ThreadPlanCallFunction::PlanExplainsStop: Got stop reason - {0}.",
+           Thread::StopReasonAsString(stop_reason));
 
   if (stop_reason == eStopReasonBreakpoint && BreakpointsExplainStop())
     return true;
@@ -301,9 +271,8 @@ bool ThreadPlanCallFunction::DoPlanExplainsStop(Event *event_ptr) {
   // then we should not consider ourselves complete.  Return true to
   // acknowledge the stop.
   if (Process::ProcessEventData::GetInterruptedFromEvent(event_ptr)) {
-    if (log)
-      log->Printf("ThreadPlanCallFunction::PlanExplainsStop: The event is an "
-                  "Interrupt, returning true.");
+    LLDB_LOGF(log, "ThreadPlanCallFunction::PlanExplainsStop: The event is an "
+                   "Interrupt, returning true.");
     return true;
   }
   // We control breakpoints separately from other "stop reasons."  So first,
@@ -312,20 +281,18 @@ bool ThreadPlanCallFunction::DoPlanExplainsStop(Event *event_ptr) {
   // m_ignore_breakpoints.
 
   if (stop_reason == eStopReasonBreakpoint) {
-    ProcessSP process_sp(m_thread.CalculateProcess());
     uint64_t break_site_id = m_real_stop_info_sp->GetValue();
     BreakpointSiteSP bp_site_sp;
-    if (process_sp)
-      bp_site_sp = process_sp->GetBreakpointSiteList().FindByID(break_site_id);
+    bp_site_sp = m_process.GetBreakpointSiteList().FindByID(break_site_id);
     if (bp_site_sp) {
       uint32_t num_owners = bp_site_sp->GetNumberOfOwners();
       bool is_internal = true;
       for (uint32_t i = 0; i < num_owners; i++) {
         Breakpoint &bp = bp_site_sp->GetOwnerAtIndex(i)->GetBreakpoint();
-        if (log)
-          log->Printf("ThreadPlanCallFunction::PlanExplainsStop: hit "
-                      "breakpoint %d while calling function",
-                      bp.GetID());
+        LLDB_LOGF(log,
+                  "ThreadPlanCallFunction::PlanExplainsStop: hit "
+                  "breakpoint %d while calling function",
+                  bp.GetID());
 
         if (!bp.IsInternal()) {
           is_internal = false;
@@ -333,25 +300,23 @@ bool ThreadPlanCallFunction::DoPlanExplainsStop(Event *event_ptr) {
         }
       }
       if (is_internal) {
-        if (log)
-          log->Printf("ThreadPlanCallFunction::PlanExplainsStop hit an "
-                      "internal breakpoint, not stopping.");
+        LLDB_LOGF(log, "ThreadPlanCallFunction::PlanExplainsStop hit an "
+                       "internal breakpoint, not stopping.");
         return false;
       }
     }
 
     if (m_ignore_breakpoints) {
-      if (log)
-        log->Printf("ThreadPlanCallFunction::PlanExplainsStop: we are ignoring "
-                    "breakpoints, overriding breakpoint stop info ShouldStop, "
-                    "returning true");
+      LLDB_LOGF(log,
+                "ThreadPlanCallFunction::PlanExplainsStop: we are ignoring "
+                "breakpoints, overriding breakpoint stop info ShouldStop, "
+                "returning true");
       m_real_stop_info_sp->OverrideShouldStop(false);
       return true;
     } else {
-      if (log)
-        log->Printf("ThreadPlanCallFunction::PlanExplainsStop: we are not "
-                    "ignoring breakpoints, overriding breakpoint stop info "
-                    "ShouldStop, returning true");
+      LLDB_LOGF(log, "ThreadPlanCallFunction::PlanExplainsStop: we are not "
+                     "ignoring breakpoints, overriding breakpoint stop info "
+                     "ShouldStop, returning true");
       m_real_stop_info_sp->OverrideShouldStop(true);
       return false;
     }
@@ -405,10 +370,11 @@ void ThreadPlanCallFunction::DidPush() {
   GetThread().SetStopInfoToNothing();
 
 #ifndef SINGLE_STEP_EXPRESSIONS
-  m_subplan_sp.reset(
-      new ThreadPlanRunToAddress(m_thread, m_start_addr, m_stop_other_threads));
+  Thread &thread = GetThread();
+  m_subplan_sp = std::make_shared<ThreadPlanRunToAddress>(thread, m_start_addr, 
+                                                          m_stop_other_threads);
 
-  m_thread.QueueThreadPlan(m_subplan_sp, false);
+  thread.QueueThreadPlan(m_subplan_sp, false);
   m_subplan_sp->SetPrivate(true);
 #endif
 }
@@ -419,9 +385,8 @@ bool ThreadPlanCallFunction::MischiefManaged() {
   Log *log(lldb_private::GetLogIfAllCategoriesSet(LIBLLDB_LOG_STEP));
 
   if (IsPlanComplete()) {
-    if (log)
-      log->Printf("ThreadPlanCallFunction(%p): Completed call function plan.",
-                  static_cast<void *>(this));
+    LLDB_LOGF(log, "ThreadPlanCallFunction(%p): Completed call function plan.",
+              static_cast<void *>(this));
 
     ThreadPlan::MischiefManaged();
     return true;
@@ -431,11 +396,10 @@ bool ThreadPlanCallFunction::MischiefManaged() {
 }
 
 void ThreadPlanCallFunction::SetBreakpoints() {
-  ProcessSP process_sp(m_thread.CalculateProcess());
-  if (m_trap_exceptions && process_sp) {
+  if (m_trap_exceptions) {
     m_cxx_language_runtime =
-        process_sp->GetLanguageRuntime(eLanguageTypeC_plus_plus);
-    m_objc_language_runtime = process_sp->GetLanguageRuntime(eLanguageTypeObjC);
+        m_process.GetLanguageRuntime(eLanguageTypeC_plus_plus);
+    m_objc_language_runtime = m_process.GetLanguageRuntime(eLanguageTypeObjC);
 
     if (m_cxx_language_runtime) {
       m_should_clear_cxx_exception_bp =
@@ -470,9 +434,8 @@ bool ThreadPlanCallFunction::BreakpointsExplainStop() {
          m_objc_language_runtime->ExceptionBreakpointsExplainStop(
              stop_info_sp))) {
       Log *log(lldb_private::GetLogIfAnyCategoriesSet(LIBLLDB_LOG_STEP));
-      if (log)
-        log->Printf("ThreadPlanCallFunction::BreakpointsExplainStop - Hit an "
-                    "exception breakpoint, setting plan complete.");
+      LLDB_LOGF(log, "ThreadPlanCallFunction::BreakpointsExplainStop - Hit an "
+                     "exception breakpoint, setting plan complete.");
 
       SetPlanComplete(false);
 
@@ -491,16 +454,15 @@ void ThreadPlanCallFunction::SetStopOthers(bool new_value) {
   m_subplan_sp->SetStopOthers(new_value);
 }
 
-bool ThreadPlanCallFunction::RestoreThreadState() {
-  return GetThread().RestoreThreadStateFromCheckpoint(m_stored_thread_state);
+void ThreadPlanCallFunction::RestoreThreadState() {
+  GetThread().RestoreThreadStateFromCheckpoint(m_stored_thread_state);
 }
 
 void ThreadPlanCallFunction::SetReturnValue() {
-  ProcessSP process_sp(m_thread.GetProcess());
-  const ABI *abi = process_sp ? process_sp->GetABI().get() : nullptr;
+  const ABI *abi = m_process.GetABI().get();
   if (abi && m_return_type.IsValid()) {
     const bool persistent = false;
     m_return_valobj_sp =
-        abi->GetReturnValueObject(m_thread, m_return_type, persistent);
+        abi->GetReturnValueObject(GetThread(), m_return_type, persistent);
   }
 }
